@@ -432,8 +432,8 @@ static apr_status_t no_more_writes(serf_connection_t *conn)
 {
     /* Note that we should hold new requests until we open our new socket. */
     conn->state = SERF_CONN_CLOSING;
-    serf__log(CONN_VERBOSE, __FILE__, "stop writing on conn 0x%x\n",
-              conn);
+    serf__log_skt(CONN_VERBOSE, __FILE__, conn->skt,
+                  "stop writing on conn 0x%x\n", conn);
 
     /* Clear our iovec. */
     conn->vec_len = 0;
@@ -916,21 +916,22 @@ static apr_status_t handle_response(serf_request_t *request,
      * themselves by not registering credential callbacks.
      */
     if (request->conn->ctx->cred_cb) {
-      status = serf__handle_auth_response(&consumed_response,
-                                          request,
-                                          request->resp_bkt,
-                                          request->handler_baton,
-                                          pool);
+        status = serf__handle_auth_response(&consumed_response,
+                                            request,
+                                            request->resp_bkt,
+                                            request->handler_baton,
+                                            pool);
 
-      /* If there was an error reading the response (maybe there wasn't
-         enough data available), don't bother passing the response to the
-         application.
+        if (SERF_BUCKET_READ_ERROR(status)) {
+            /* Report the request as 'died'/'cancelled' to the application */
+            (void)(*request->handler)(request,
+                                      NULL,
+                                      request->handler_baton,
+                                      pool);
+        }
 
-         If the authentication was tried, but failed, pass the response
-         to the application, maybe it can do better. */
-      if (status) {
-          return status;
-      }
+        if (status)
+            return status;
     }
 
     if (!consumed_response) {
@@ -1285,8 +1286,29 @@ apr_status_t serf__process_connection(serf_connection_t *conn,
                 int error;
                 apr_socklen_t l = sizeof(error);
 
-                if (!getsockopt(osskt, SOL_SOCKET, SO_ERROR, (char*)&error, &l))
-                    return APR_FROM_OS_ERROR(error);
+                if (!getsockopt(osskt, SOL_SOCKET, SO_ERROR, (char*)&error,
+                                &l)) {
+                    status = APR_FROM_OS_ERROR(error);
+
+                    /* Handle fallback for multi-homed servers.
+                     
+                       ### Improve algorithm to find better than just 'next'?
+
+                       Current Windows versions already handle re-ordering for
+                       api users by using statistics on the recently failed
+                       connections to order the list of addresses. */
+                    if (conn->completed_requests == 0
+                        && conn->address->next != NULL
+                        && (APR_STATUS_IS_ECONNREFUSED(status)
+                            || APR_STATUS_IS_TIMEUP(status)
+                            || APR_STATUS_IS_ENETUNREACH(status))) {
+
+                        conn->address = conn->address->next;
+                        return reset_connection(conn, 1);
+                    }
+
+                    return status;
+                  }
             }
         }
 #endif
